#!/usr/bin/env python3
import os
import sys
# import random
import time
# from random import seed, randint
# import argparse
import platform
# from datetime import datetime
# import imp
# from time import sleep
# import fileinput
# from myfunction import *

# from helperFunctions.myFunctions import *

from simtk.openmm.app import *
from simtk.openmm import *
from simtk.unit import *

import numpy as np
from sys import stdout


def test_term(pdb):
    contact = CustomGBForce()
    gamma_ijm = np.zeros((2, 20, 20))
    # if you comment out the line below, then no error will occur
    contact.addTabulatedFunction("gamma_ijm", Discrete3DFunction(2, 20, 20, gamma_ijm.T.flatten()))
    contact.addComputedValue("rho", f"1", CustomGBForce.ParticlePair)
    contact.addComputedValue("alpha", f"z", CustomGBForce.SingleParticle)
    for i in range(pdb.topology.getNumAtoms()):
        contact.addParticle()
    return contact

# if use CPU, then no error, but if I use OpenCL or CUDA then there is a problem.
simulation_platform = "OpenCL"
# simulation_platform = "CUDA"
# simulation_platform = "CPU"
platform = Platform.getPlatformByName(simulation_platform)
if simulation_platform == "CPU":
    print(f"{simulation_platform}: {platform.getPropertyDefaultValue('Threads')} threads")

proteinName = pdb_id = "1r69"
# chain="A"
# pdb = f"{pdb_id}.pdb"
input_pdb_filename = f"{pdb_id}-openmmawsem.pdb"
reporter_frequency = 100


pdb = PDBFile(input_pdb_filename)
forcefield = ForceField("awsem.xml")
system = forcefield.createSystem(pdb.topology)
system.addForce(test_term(pdb))
# with open('system.xml', 'w') as output:
#   output.write(XmlSerializer.serialize(system))

# start simulation
collision_rate = 5.0 / picoseconds

integrator = LangevinIntegrator(600*kelvin, 1/picosecond, 2*femtoseconds)
simulation = Simulation(pdb.topology, system, integrator, platform)
simulation.context.setPositions(pdb.positions) # set the initial positions of the atoms
# simulation.context.setVelocitiesToTemperature(300*kelvin) # set the initial velocities of the atoms according to the desired starting temperature
simulation.minimizeEnergy() # first, minimize the energy to a local minimum to reduce any large forces that might be present
simulation.reporters.append(StateDataReporter(stdout, reporter_frequency, step=True, potentialEnergy=True, temperature=True)) # output energy and temperature during simulation
# simulation.reporters.append(PDBReporter("movie.pdb", reporter_frequency))  # output PDBs of simulated structures


print("Simulation Starts")
start_time = time.time()


simulation.step(int(1e3))



time_taken = time.time() - start_time  # time_taken is in seconds
hours, rest = divmod(time_taken,3600)
minutes, seconds = divmod(rest, 60)
print(f"---{hours} hours {minutes} minutes {seconds} seconds ---")
